__author__ = 'Slaven Glumac'
__email__ = 'slaven.glumac@gmail.com'
__version__ = '0.1.2'

import pyislands.types
import pyislands.ga
import pyislands.evolution
import pyislands.creation
import pyislands.archipelago

import pyislands.permutation
import pyislands.selection

import pyislands.example
