'''
Module contains mutation generators for permutation mutations.
Mutation is function with signature:

    def mutation(genotype):
        ...
        return mutated_genotype

'''
import pyislands.permutation.mutation.ecm
import pyislands.permutation.mutation.rsm
